/*
 * Copyright (c) Solexa 2004, 2005, 2006
 * Author: 
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */
//---------------------------------------------------------------------------

#ifndef ImageProcessH
#define ImageProcessH
//---------------------------------------------------------------------------

#include <stdio.h>
#include "ImageUtil.h"
#include "ProcessObjectDetection.h"
#include "ProcessDetectedObjectStats.h"
#include "ProcessAlign.h"
#include "ParameterTypes.h"

const float image_default_fwhm      = 2.7f;
const float image_default_distance  = 1.5f;
const float image_default_threshold = 4.f;

const AnalysisParameters image_default_parameters = {
    image_default_fwhm, image_default_distance, image_default_threshold, 0., 0.
};

struct ImageStatsRecord : public ImageStats {
    ImageRegion Region;
};

struct ImageAnalysisRecord {
    float average_noise;
    float average_threshold;
    float average_threshold_signal;
    float average_threshold_background;
};

struct SequencingRecord {
    unsigned char base;
    int is_deblocked;
};

#define IMAGEDATA_STATE_IMAGE_STATS      (1<<0)
#define IMAGEDATA_STATE_FILTER           (1<<1)
#define IMAGEDATA_STATE_BACKGROUND_NOISE (1<<2)
#define IMAGEDATA_STATE_ANALYSED         (1<<3)
#define IMAGEDATA_STATE_OBJ_STATS        (1<<4)
#define IMAGEDATA_STATE_FILTER_OBJ_STATS (1<<5)

class ImageData {
  public:
    float *image;
    int nx, ny;
    AnalysisParameters analysis_params;
    unsigned short bitsperpixel;
    int filter_type;
    float *fourier_image;
    int fnx, fny;
    float *filter_image;
    float *noise_image;
    float *background_image;
    st_object *objects;
    ObjectStatsRecord *filtered_object_stats;
    ObjectStatsRecord *object_stats;
    int nobj;
    ClusterStats cluster_stats;
    st_pixel *positions;
    st_fnoisepixel *positiondata;
    int status;
    ImageStatsRecord *image_stats;
    ImageStatsRecord *local_image_stats;
    int nlocal_stats;
    ImageAnalysisRecord *analysis_stats;
    SequencingRecord *sequencing;
    AlignmentTransform *transform;
    AlignmentTransform *transform2;
    AlignmentTransform *usedTransform;
    float estimated_fwhm;
  private:
    int nmax;
  public:
    ImageData(unsigned short *data, int x, int y, unsigned short bpp);
    ImageData(float *data, int x, int y, unsigned short bpp);
    virtual ~ImageData();
    void setParameters(const AnalysisParameters &params);
    void setSequencingInfo(unsigned char base, int is_deblocked = 0);
    void setAlignmentTransform(AlignmentTransform *t) { transform = t; }
    void analyseImageStats();
    void analyseLocalImageStats();
    void analyseFilteredObjectStats();
    void analyseObjectStats();
    void analyseFilter();
    void analyseBackgroundNoise();
    void analyseEstimateFWHM();
    void analyse();
    void clear();
    void clearImages();
    int writeObjectStats(const char filename[],
                         st_fnoisepixel *lists = 0,
                         int *indices = 0, int count = 0);
    int writePositions(FILE *filehandle, int indexflag = 0);                      
};
    
#endif /* ImageProcessH */
