
/*
 *
 * Copyright (c) 2006 Solexa Ltd.
 * Author: Lorenzo Milazzo
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 *
 */

//
// CVS - $Id: distributional_adequacy.cpp,v 1.1 2006/10/24 13:00:52 lm Exp $
//



#include <cmath>

#include "distributional_adequacy.h"


int getIndex(float data1_el_s, float *data2, int no_el2)
{
    int i, il, inew, iu;


    if (data1_el_s > data2[(no_el2 - 1)])
        return (no_el2 - 1);

    if (data1_el_s < data2[0])
        return (-1);

    il = 0;
    iu = (no_el2 - 1);
    i = (il + iu) / 2;

    while (1) {
        if (data2[i] >= data1_el_s) {
            inew = (i + il) / 2;
            iu = i;
            if (inew == i)
                return (i - 1);
        }
        else {
            inew = (i + iu) / 2;
            il = i;
            if (inew == i)
                return (i);
        }
        i = inew;
    }


}


//
// scaled cumulative distribution function (scaled cdf)
//
float scaledCumulDistrFunc(float x, float data1_el, float *data2, int no_el2)
{
    int index;
    float data1_el_s;
    float ff;


    data1_el_s = data1_el * x;

    index = getIndex(data1_el_s, data2, no_el2);

    ff = (float)(index + 1) / no_el2;

    return (ff);


}


//
// KS statistic function
// (Kolmogorov-Smirnov statistic, D-statistic)
//
//  x = scale factor (independent variable)
//  d = KS statistic
//
float KSStatisticFunc(float x, void *p_args_ks_v)
{
    int i;
    float d, dt, ff, fn, fo;

    struct args_ks *p1;

    p1 = (struct args_ks *)p_args_ks_v;

    d = 0.0;
    fo = 0.0;

    for (i = 0; i < p1->no_el1; i++) {
        fn = (float)(i + 1) / (p1->no_el1);
        ff = (*p1->p_func) (x, p1->data1[i], p1->data2, p1->no_el2);
        dt = FMAX(fabs(fo - ff), fabs(fn - ff));
        if (dt > d)
            d = dt;
        fo = fn;
    }

    return (d);


}
