/*
 * Copyright (c) 2006-2007 Solexa
 *               2007      Illumina
 * Authors: Lorenzo Milazzo
 *          Klaus Maisinger
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 *
 */

#include <string>
#include <vector>
#include <cassert>

#include "input_process.h"
#include "savinio_common.h"
#include "IntensityData.h"

#include "savinio_debugging.h"


InputProcess::InputProcess(const char *input_file) : 
     filename(input_file)
{
}

//
// input file:       intensity file (Firecrest intensity output file)
//
// the intensity file consists of four intensity values per cluster (corresponding
// to the uncorrected A, C, G, T channels in this order), one line per cluster;
// it contains the intensities associated with all clusters, for a given cycle,
// for a given tile, for a given lane;
//
void InputProcess::fromInput_Intensities(float     * intensities, 
                                         int       & no_objects,
                                         const int   no_cols,
                                         const int   compression,
                                         const int   cycle)
{
    int n_cols, n_clusters, n_filt_clusters, n_skip_cols;
    n_clusters      = MAX_NO_CLUSTERS;
    n_cols          = no_cols;
    n_filt_clusters = 0;
    n_skip_cols     = 0;

    if (cycle >= 0)
        n_skip_cols = 4 + cycle * no_cols;

    read_signal_array(filename.c_str(), intensities, &n_clusters, &n_cols,
                      compression, n_skip_cols);
    assert((no_cols == n_cols) || (n_cols == 0));
    no_objects = n_clusters;
}
