/*
 * Time-stamp: <26 Jun 2006 17:04 BST ProcessAlign.h km>
 * Copyright (c) Solexa 2005, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

//---------------------------------------------------------------------------

#ifndef _PROCESSALIGN_H
#define _PROCESSALIGN_H

#include "ImageUtil.h"
#include "ParameterTypes.h"

struct AlignmentRecord : public AlignmentStats {
public:
    virtual ~AlignmentRecord() {};
};

int validateAlignment(AlignmentRecord *transforms, int ntrans,
                      float max_offset = 2., int includeflag = -1);

void align_images(float *image1, float *image2,
                  int nx, int ny, int xmaxlag, int ymaxlag,
                  AlignmentRecord *alignment);
void align_images_fft(float *image1, float *image2,
                      int nx, int ny, int xmaxlag, int ymaxlag,
                      AlignmentRecord *alignment);
void align_fft_images(float *image1, float *image2, int nx, int ny, 
                      int xmaxlag, int ymaxlag,
                      AlignmentRecord *alignment);
void create_clean_fft_image(float *fourier, int nx, int ny,
                            st_pixel *positions, int npos,
                            float fwhm, int uniform);
void align_positions(st_pixel *positions1, int n1,
                     st_pixel *positions2, int n2,
                     int nx, int ny, float fwhm, int xmaxlag, int ymaxlag,
                     AlignmentRecord *alignment);
int calculate_object_overlap(st_pixel *positions1, int n1,
                             st_pixel *positions2, int n2,
                             int nx, int ny, int xoff, int yoff,
                             float min_dist);
int calculate_object_overlap(st_fnoisepixel *positions1, int n1,
                             st_fnoisepixel *positions2, int n2,
                             int nx, int ny, int xoff, int yoff,
                             float min_dist);

class AlignmentTransform : public AlignmentRecord
{
 public:
    AlignmentRecord *local_alignment;
    int nlocal;
    AlignmentTransform(int x = 0, int y = 0)
        { XOffset = 0.f; XOffset = 0.f;
          XScale = 0.f;  YScale = 0.f;
          Region.XMin  = 0; Region.YMin  = 0;
          Region.XSize = x; Region.YSize = y;
          CorrelationMinimum = 0.f; CorrelationMaximum = 0.f;
          CorrelationWidth = 0.f;
          local_alignment = 0; nlocal = NumAlignmentRegions; }
    AlignmentTransform(const AlignmentTransform &ref)
    {
        AlignmentTransform();
        copy(ref);
    }
    ~AlignmentTransform()
    {
        if (nlocal > 0)
            delete [] local_alignment;
    }
    void copy(const AlignmentTransform & ref);
    void copy(const Alignment & ref, int nx, int ny);
    void transformForward(float *x, float *y) const
    {
        *x += (*x - Region.XSize/2) * XScale + XOffset;
        *y += (*y - Region.YSize/2) * YScale + YOffset;
    }
    void transformReverse(float *x, float *y) const
    {
        *x = (*x + Region.XSize/2.f*XScale - XOffset) / (1.f+XScale);
        *y = (*y + Region.YSize/2.f*YScale - YOffset) / (1.f+YScale);
    }
    void transformReverse(st_fnoisepixel *pixel, int n) const;
};

int loadAlignmentTransforms(const char filename[],
                            Alignment *transforms, int nmax);

class ProcessAlign
{
    float *reference_fourier_image;
    st_pixel *reference_positions;
    int reference_count;
    float fwhm;
    int maxlagx, maxlagy;
    int nx, ny, fnx, fny;
    int numregionx, numregiony;
    int regionsizex, regionsizey;
 public:
    ProcessAlign(st_fposition *refpositions, int refcount,
                 int nx2, int ny2, float fwhm);
    ~ProcessAlign();
    void setMaximumOffsets(int xlag, int ylag = -1);
    void alignPixelPositions(st_fnoisepixel *positions, int npos,
                             AlignmentRecord *offsets, float fwhm = -1.f);
    void setRegionParameters(int numx, int numy, int sizex = -1, int sizey = -1); 
    void alignRegionPixelPositions(st_fnoisepixel *positions, int npos,
                                   AlignmentTransform *transform);
};

#endif /* _PROCESSALIGN_H */
