/*
 * Detect objects on a thresholded image
 *
 * Copyright (c) Solexa 2005, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */
#ifndef _PROCESSOBJECTDETECTION_H_
#define _PROCESSOBJECTDETECTION_H_

#include "ImageUtil.h"

#define MIN_OBJ_PIXELS 3

struct st_object {
    struct st_pixel    * boundary;
    int                  nb;
    struct st_pixel    * area;
    int                  na;
    struct st_pixel      max_pixel;
    struct st_position   min, max;
    int                  is_split;
    st_fposition         position;
};

void free_object(struct st_object *obj);

void interpolate_objects(float            * array, 
                         int                nx,
                         int                ny,
                         struct st_object * objects,
                         int                nobj);

void identify_objects(float            * spatial,
                      int                nx,
                      int                ny,
                      struct st_object * objs,
                      int                nmax,
                      int              * nobjs,
                      unsigned short     min_obj_size = MIN_OBJ_PIXELS,
                      float              threshold = 0.);
void detect_objects(float            * filter_image,
                    float            * image,
                    int                nx,
                    int                ny, 
                    float            * noise,
                    float              factor,
                    float              fwhm,
                    float              min_dist,
                    struct st_object * areas,
                    int                nmax,
                    int              * nareas);

#endif //_PROCESSOBJECTDETECTION_H_
