/*
 * Copyright (c) 2007 Illumina
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#include "XMLImageConfigHandler.h"
#include "ImageProcess.h"

XMLImageConfigHandler::XMLImageConfigHandler() : inParameters(0), inFwhm(0),
                                                 inThreshold(0), inDistance(0)
{
    params.DetectionThreshold = image_default_threshold;
    params.ClusterFwhm        = image_default_fwhm;
    params.ClusterDistance    = image_default_distance;
}

XMLImageConfigHandler::~XMLImageConfigHandler()
{
}

void XMLImageConfigHandler::startElement(const char name[], const Attributes *attr)
{
    std::string tag(name);
    if (tag == "ImageParameters")
        inParameters = 1;
    if (tag == "Fwhm")
        inFwhm = 1;
    if (tag == "Threshold")
        inThreshold = 1;
    if (tag == "RemappingDistance")
        inDistance = 1;
}

void XMLImageConfigHandler::characters(const char ch[], int start, int length)
{
    if (! inParameters)
        return;
    if (inFwhm)
        params.ClusterFwhm        = atof(ch);
    if (inDistance)
        params.ClusterDistance    = atof(ch);
    if (inThreshold)
        params.DetectionThreshold = atof(ch);
}

void XMLImageConfigHandler::endElement(const char name[])
{
    std::string tag(name);
    if (tag == "ImageParameters")
        inParameters = 0;
    if (tag == "Fwhm")
        inFwhm = 0;
    if (tag == "Threshold")
        inThreshold = 0;
    if (tag == "RemappingDistance")
        inDistance = 0;
}

void XMLImageConfigHandler::getParameters(struct AnalysisParameters &params2)
{
    params2.ClusterFwhm        = params.ClusterFwhm;
    params2.ClusterDistance    = params.ClusterDistance;
    params2.DetectionThreshold = params.DetectionThreshold;
}

