#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  buildExport.pl
# AUTHOR:  A. J. Cox
#
# Copyright (c) 2007 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Build a tab delimited file designed for export to database

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: buildExport.pl,v 1.3 2007/11/06 09:24:04 tc Exp $';
    my $cvs_version='$Revision: 1.3 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


use warnings;
use strict;
use File::Basename;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;

my $numericQuality=undef;
my $shortFormat=undef;
my $parseLine=\&parseLineSingle;

my ($readID, $seqString, $qualString, $dummy, $matchChrom, $matchPos,
    $matchStrand, $matchDescriptor, 
    $singleReadAlignmentScore, $pairedReadAlignmentScore,
    $partnerStrand, $partnerOffset);

my ($filtered,$machineName,$runNumber,$lane,$tile,$x,$y,$readNum,$index,
    $matchContig,$partnerChrom,$partnerContig);

my @F;


my $usage=<<"END";
Usage: paste s_6_seqpre.txt s_6_qcal.txt s_6_calsaf.txt s_6_filt.txt | $0 
[--paired] [--numeric|--symbolic] [--short]
END

if ((@ARGV>0)&&($ARGV[0] eq '--paired'))
{
    $parseLine=\&parseLinePaired;
    shift;
} # if

if (@ARGV>0)
{
    if ($ARGV[0] eq '--numeric')
    {
	$numericQuality=1;
	shift;
    } # if
    elsif ($ARGV[0] eq '--symbolic')
    {
	shift;
    } # elsif
} # if

if ((@ARGV>0)&&($ARGV[0] eq '--short'))
{
    $shortFormat=1;
    shift;
} # if

die $usage unless (@ARGV==0);


sub explodeReadID($)
{
    my ($readNum,$index)=("","");
    if ($_[0]=~s/\/([12])$//)
    {
	$readNum=$1;
    }
    if ($_[0]=~s/\#(.*?)$//)
    {
	$index=$1;
    }

    my ($readID,$lane,$tile,$x,$y)=split(":",$_[0]);
    my @a=split('_',$readID);
    if (@a!=2)
    {
	$a[0]=$readID unless (@a==1);
	$a[1]="";
    }

    return "$a[0]\t$a[1]\t$lane\t$tile\t$x\t$y\t$index\t$readNum";
}

# return chrom and optional contig
sub explodeChrom($)
{
    $_[0]=~/^(.*?)(\/(.*))?$/;
    my ($chrom,$contig)=($1,$3);
    $chrom="" unless (defined($chrom));
    $contig="" unless (defined($contig));
    return "$chrom\t$contig";
}

sub explodePartnerOffset($)
{
    $_[0]=~/((.*?)(\/(.*))?:)?(\-?\d+)$/; 
    my ($chrom,$contig,$offset)=($2,$4,$5);
    $chrom="" unless (defined($chrom));
    $contig="" unless (defined($contig));
    $offset="" unless (defined($offset));
    return "$chrom\t$contig\t$offset";
}

sub parseLinePaired($)
{
    chomp;
    my ($readID, $seqString, $qualString, $dummy, $matchChrom, $matchPos,
     $matchStrand, $matchDescriptor, 
     $singleReadAlignmentScore, $pairedReadAlignmentScore,
     $partnerStrand, $partnerOffset,$filtered)=split("\t",$_[0]);

    unless (defined($filtered))
    {
	$filtered=$matchPos;
	$matchPos=$matchStrand=$matchDescriptor=$singleReadAlignmentScore
	    =$pairedReadAlignmentScore=$partnerStrand=$partnerOffset="";
    } # unless
    return
    ($readID, $seqString, $qualString, $matchChrom, $matchPos,
     $matchStrand, $matchDescriptor, 
     $singleReadAlignmentScore, $pairedReadAlignmentScore,
     $partnerStrand, $partnerOffset,$filtered);
} # parseLinePaired

sub parseLineSingle($)
{
    chomp;
    my ($readID, $seqString, $qualString, $dummy, $matchChrom, $matchPos,
     $matchStrand, $matchDescriptor, 
     $singleReadAlignmentScore, $filtered)=split("\t",$_[0]);
    my ($pairedReadAlignmentScore,$partnerStrand,$partnerOffset)
	=("","","");

    unless (defined($filtered))
    {
	$filtered=$matchPos;
	$matchPos=$matchStrand=$matchDescriptor=$singleReadAlignmentScore
	    ="";
    } # unless

    return
    ($readID, $seqString, $qualString, $matchChrom, $matchPos,
     $matchStrand, $matchDescriptor, 
     $singleReadAlignmentScore, $pairedReadAlignmentScore,
     $partnerStrand, $partnerOffset,$filtered);
} # parseLinePaired



while (<STDIN>)
{
    chomp;
    ($readID, $seqString, $qualString, $matchChrom, $matchPos,
     $matchStrand, $matchDescriptor, 
     $singleReadAlignmentScore, $pairedReadAlignmentScore,
     $partnerStrand, $partnerOffset,$filtered)=$parseLine->($_);


    unless (defined($shortFormat))
    {
	$readID=explodeReadID($readID);
	$matchChrom=explodeChrom($matchChrom);
	$partnerOffset=explodePartnerOffset($partnerOffset);
    } # unless
    if (defined($numericQuality))
    {
	$qualString=expandSymbolicQualityString($qualString)
    } # if


    print "$readID\t$seqString\t$qualString\t$matchChrom\t$matchPos\t$matchStrand\t$matchDescriptor\t$singleReadAlignmentScore\t$pairedReadAlignmentScore\t$partnerOffset\t$partnerStrand\t$filtered\n";
}
