#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  byTileThumbnailView.pl
# AUTHOR:  A. J. Cox
#
# Copyright (c) 2006 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description: look for all png files in a directory that seem to be 
# associated with an individual tile and print them out as thumbnails,
# one tile per row

use warnings;
use strict;
use Cwd;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: byTileThumbnailView.pl,v 1.4 2006/07/27 13:59:46 tc Exp $';
    my $cvs_version='$Revision: 1.4 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope



my ($file,$i,$j,$k,$exptName,$tileName, $imageName);

my %lane;
my %tile;
my %type;

$exptName="";

die "Usage: $0 [dir]" 
    unless (@ARGV<2);

$ARGV[0]=getcwd() unless (@ARGV==1);


opendir (DIR,$ARGV[0]) or die "can't open $ARGV[0]: $!";

while (defined($file = readdir (DIR)))
{
#    print "$file";
    if ($file=~/^s_([1-8])_(\d{4})_(.+)\.png$/)
    {
#        print " ... is wanted";
        $lane{$1}++;
        $tile{$2}++;
        $type{$3}++;
    } # if
#    print "\n";
} # while

my @typeList=sort keys %type;


## Define output file name


print  "<html>\n<head>\n";

print  "<!--RUN_TIME ".scalar(localtime)."-->\n";
print  "<!--SOFTWARE_VERSION $id-->\n";


#print  qq(</head>)."\n";

print  qq(<title>$exptName</title>);
print  qq(<h1 align="center">$exptName</h1>);


print "\n<body>\n";



## Starting the html output with the body name

print "<table><tr>\n";
foreach $k (sort {$a<=>$b} keys %lane)
{
    print qq(<td><a href="#$k"><b>Lane $k</b></a></td>)."\n";
}
print "</tr></table>\n";

print  qq(<table border="1" cellpadding="5"><tr><td><b>Tile</b></td>)."\n";
foreach $i (@typeList)
{
    print  qq(<td><b> $i</b></td>)."\n";
} # for i 
print  qq(</tr>)."\n";
foreach $k (sort {$a<=>$b} keys %lane)
{
    print  qq(<tr><td><a name="$k"><b>Lane $k</a></a></td></tr>)."\n";   
    foreach $j (sort {$a<=>$b} keys %tile)
    {
        $tileName=sprintf "%4.4d", $j;
        print "<tr><td><b>s_${k}_${tileName}</td>\n";
        foreach $i (@typeList)
        {
            $imageName="s_${k}_${tileName}_${i}.png";
            print  qq(<td><a href="$imageName"> <img height=84 width=84 src="$imageName"></a></td>)."\n";
        } # for i 
        print  "</tr>\n";
    } # for j
} # for i





print qq(</table>)."\n";
print qq(</body>)."\n";
print qq(</html>)."\n";


