#!/usr/bin/env perl
use warnings;
use strict;
use File::Basename;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;

# File: convertToFasta.pl
# Copyright (c) Solexa Limited 2006-7
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Simple script to convert "Solexa-style" sequence files into
# Fasta-style format

# Usage: 
# [ cat seq1.txt seq2.txt... ] | ./convertToFasta.pl 
# [-useBases (all|odd|YNYN...) ] [ seq3.txt seq4.txt ... ]

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: convertToFasta.pl,v 1.10 2007/10/03 10:47:45 tc Exp $';
    my $cvs_version='$Revision: 1.10 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope



my $seqFile;
#my $seqFileNoPath;
my $seq;
my $useBases="all";
my $indexBases=undef;
#my $unp=undef;
my @s;
my $l;
my $seqIn;
my $seqOut;
# indexOut defaults to empty string so if indexing not done it just gets
# printed without harm
my $indexOut="";
my $shortRunFolderName;
my $runFolderWithIndex;
my $readID;


# $readPairMode==0 (default) denotes no pairing - treat 'Y' and 'y' the same
# $readPairMode==1 denotes pair1 - ignore 'y' bases
# $readPairMode==2 denotes pair2 - ignore 'Y' bases
my $readPairMode=0;

my @basesForAnalysis;
my @basesForIndexing;
my $readLength;
my $indexLength=undef;


my $i;
my $j;

sub setUpUseBases
{
#    $readLength=length ($seqIn);
    $readLength=((($useBases eq 'all')||($useBases eq 'odd'))
		 ?length($seqIn):0);
    @basesForAnalysis=parseUseBases
	($useBases,$readLength,$readPairMode);
    $readLength=scalar(@basesForAnalysis);
    $seqOut='X' x $readLength;
    if (defined($indexBases))
    {
	@basesForIndexing=parseUseBases
	    ($indexBases,0);
	$indexLength=scalar(@basesForIndexing);
	$indexOut= 'X' x $indexLength;
	# check useBases and indexBases are consistent
	my %h;
	for ($i=0;$i<$readLength;$i++)
	{
	    $h{$basesForAnalysis[$i]}++;
	}
	for ($i=0;$i<$indexLength;$i++)
	{
	    $h{$basesForIndexing[$i]}++;
	}
	for $i (sort {$a<=>$b} keys %h)
	{
	    die "Error: use bases $useBases and index bases $indexBases both specify cycle ", $i+1
		unless ($h{$i}==1);
	} # for

    } # if
} # setUpUseBases


if ((@ARGV>=2)&&($ARGV[0] eq "-useBases"))
{
    shift (@ARGV);
    $useBases=shift(@ARGV);
} # if


if ((@ARGV>0)&&($ARGV[0]=~/^\-read([012])$/))
{
    $readPairMode=$1;
    shift (@ARGV);
} # if

if ((@ARGV>=2)&&($ARGV[0] eq "-indexBases"))
{
    shift (@ARGV); 
    $indexBases=shift(@ARGV);
} # if


my $usage=<<"END";
Usage: $0
[-useBases (all|odd|YNYN...)] 
[-read[1|2]] 
[-indexBases (all|odd|YNYN...)] 
seq1.txt [ seq2.txt... ]
END

die $usage unless (@ARGV>0);

$"=':';

#$"='_';

while ($seqFile=shift(@ARGV))
{
    open (FILE, $seqFile)
        || die "Problem opening $seqFile: $!";

    my $bustardFolder=File::Spec->rel2abs(dirname($seqFile));
    @s=split('/',$bustardFolder);
  #  print scalar(@s), "$bustardFolder\n"; die;
    my $shortRunFolderName=getShortRunFolderName($s[-4]); 

#    $seqFileNoPath=basename($seqFile);

    while ($l=<FILE>)
    {
#    print $l;
        chomp($l);
        @s=split(' ',$l);
        $seqIn=pop(@s);

        if (@basesForAnalysis==0)
        {
	    setUpUseBases();
        } # if

	for ($i=0;$i<$readLength;$i++)
	{
	    substr($seqOut,$i,1)
		=substr($seqIn,$basesForAnalysis[$i],1);
	    $seqOut=~s/\./N/g;
	} # for

	if (defined($indexLength))
	{
	    for ($i=0;$i<$indexLength;$i++)
	    {
		substr($indexOut,$i,1)
		    =substr($seqIn,$basesForIndexing[$i],1);
	    } # for
#	    $indexOut=~s/\./N/g;
#	    $runFolderWithIndex=$shortRunFolderName;
#	    $runFolderWithIndex.='/';
#	    $runFolderWithIndex.=$indexOut;
	    $readID=getReadID($shortRunFolderName,
			  $s[0],$s[1],$s[2],$s[3],$readPairMode,$indexOut);
	} # if
	else
	{
	    $readID=getReadID($shortRunFolderName,
			  $s[0],$s[1],$s[2],$s[3],$readPairMode);
	} # else

	print "${readID}\t${seqOut}\n";

#        $seqOut="";
#        map {$seqOut.=chr($_) } unpack $unp, $seqIn;
#        $seqOut=~s/\./N/g;
#	$readID=getReadID($shortRunFolderName,
#			  $s[0],$s[1],$s[2],$s[3],$readPairMode);
#        print ">${readID}\n${seqOut}\n";
#        print ">${shortRunFolderName}:@s\n$seqOut\n";
  
    } # while FILE...

    close (FILE);

} # while $seqFile...
