#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  filterQualityValues.pl
# AUTHOR:  A. J. Cox
#
# Copyright (c) 2006 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Quality values output by the quality value recalibration code
# include values for all cycles. Need to filter out any cycles specified
# as 'N' in the USE_BASES string.

use warnings;
use strict;
use File::Basename;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;


my $version;
my $id;
{
    my $cvs_id='@(#) $Id: filterQualityValues.pl,v 1.4 2007/06/12 15:27:34 tc Exp $';
    my $cvs_version='$Revision: 1.4 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


my $usage=
"Usage: cat qualityValues.txt | $0 useBases [--numeric|--symbolic] readLength";


my $useBases=undef;
my $isNumeric=0;

die $usage unless (@ARGV>=2);

#if ($ARGV[0] eq '--useBases')
#{
#    shift (@ARGV);
#    die $usage unless (@ARGV>=3);
$useBases=shift(@ARGV); 

#} # if

if (($ARGV[0] eq '--numeric')||($ARGV[0] eq '--symbolic'))
{
    $isNumeric=1 if ($ARGV[0] eq '--numeric');
    shift (@ARGV);
}

die $usage unless (@ARGV==1);

my $readLength=shift(@ARGV);


my @baseIndex;

unless (defined($useBases))
{
    $useBases='all';
}

my $i;
my $j;
my $qv;

my $qstring;
my $qout;




sub max
{
    my $max=$_[0]->[0];
    for (@{$_[0]})
    {
        $max=$_ if ($_>$max);
    }
    return $max;
} # sub max

# short cut if no filtering needed
# Removed! TC 31.1.7 - need to truncate to read length to build sequences correctly
#if ($useBases eq 'all')
#{
#    while (<STDIN>)
#    {
#	print;
#    } # while
#    exit (0);
#} # if


#print "$useBases\n"; die;
@baseIndex=parseUseBases($useBases, $readLength);


#print scalar(@baseIndex), " $baseIndex[0] $baseIndex[-1]\n"; die;
my @s1;
my @s3;

if ($isNumeric==0)
{ # symbolic mode
    while ($qstring=<STDIN>)
    {
	chomp($qstring);
	$qout="";
	for ($i=0;$i<scalar(@baseIndex);$i++)
	{
#            print "$i $baseIndex[$i] $s1[$baseIndex[$i]]\n";
	    unless ($baseIndex[$i]<length($qstring))
	    {
		print STDERR basename($0), ": Error parsing line: $_";
		print STDERR basename($0),
		": can't find quality values for cycle ",
		$baseIndex[$i]+1, "- only ",length($qstring), 
		" cycles of quality values present in $qstring\n";
		exit (-1);
	    } # else			
	    $qout.=substr($qstring,$baseIndex[$i],1);
	} # for
	print "${qout}\n";
    } # while
} # if
else
{ # numeric mode
    while (<STDIN>)
    {
	@s1=split(' ',$_);
	for ($i=0;$i<scalar(@baseIndex);$i++)
	{
#            print "$i $baseIndex[$i] $s1[$baseIndex[$i]]\n";
	    unless (defined($s1[$baseIndex[$i]]))
	    {
		print STDERR basename($0), ": Error parsing line: $_";
		print STDERR basename($0),
		": can't find quality values for cycle ",
		$baseIndex[$i]+1, "- only ",scalar(@s1), 
		" cycles of quality values present on line\n";
		exit (-1);
	    } # else			
	    print $s1[$baseIndex[$i]];
	    print " " unless ($i==@baseIndex-1);
	} # for
	print "\n";
    } # while
}

