#!/usr/bin/env perl

# File: plotCoverage.pl
# Copyright (c) 2005, 2006 Solexa
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description:
# Produces coverage plots from files output from buildCoverage.pl
# Assumes filenames end in .txt.
# For each fileName.txt, puts an associated fileName.png in same directory

use warnings;
use strict;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plotCoverage.pl,v 1.6 2007/09/19 18:02:36 km Exp $';
    my $cvs_version='$Revision: 1.6 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
    $id=~s/\$//g;
} #scope


my $smoothThresh=10000;


sub draw_graphs {
    my ($outName, $inName, $numLines, $maxCov) = @_;
    my $tmpName="/tmp/tmp-plotCoverage-$>-$$.txt";
    if ($maxCov == 0) {
	$maxCov = 1;
    }
    my $smooth="";
    $smooth=" smooth bezier " if ($numLines>$smoothThresh);

    open (OUT, ">$tmpName") || die "$tmpName";

    print OUT << "END";
#    set nokey;
    set xrange[1:$numLines];
    set yrange[0:$maxCov];
#    set output "$tmpName.png";
#    set terminal png color;
    set size 1.4,0.6;
    set terminal postscript portrait color solid;
    set title "Coverage plot of file $inName" -1;
    set xlabel "Genome position (bp)" -1;
    set ylabel "Coverage (bp)";
    plot '$inName' using 1:9 $smooth title 'error' with lines lw 2, '$inName' using 1:8 $smooth title 'correct' with lines lw 2;
END

    close (OUT);
    gnuplotImage($tmpName, $outName);
    unlink $tmpName;
}    

die "Usage: $0 cov1.txt [ cov2.txt ... ] "
    unless (@ARGV>=1);
my @covFiles = @ARGV;

foreach my $name (@covFiles) {
    my $numLines=0;
    my $maxCov=0;

    open (IN, $name) || die "Failed to open $name: $!";
    while (<IN>)
    {
        next if (/^\#/);
        $numLines++;
        my @F=split;
        die "Could not parse line $numLines: $_\n" unless (@F==9);
        $maxCov+=($F[7]+$F[8]);
    } #while
    close (IN);
    $maxCov/=$numLines;
    $maxCov*=2;
    my $outName=$name;
    $outName=~s/\.txt/\.png/;
    die if ($outName eq $name);
    draw_graphs($outName, $name, $numLines, $maxCov); 
} #while


