#!/usr/bin/env perl

# File: plotIntensity.pl
# Copyright (c) 2006 Solexa
# Author: A. J. Cox, L. J. Davies
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description:
# Produces an example intensity by cycle plot from an intensity 

use warnings;
use strict;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plotIntensity_for_IVC.pl,v 1.8 2007/08/17 14:57:00 km Exp $';
    my $cvs_version='$Revision: 1.8 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
    $id=~s/\$//g;
} #scope


my $tmpName;
my $tmpNameData;
my ($i,$j,$k,$outName,$displayName,$total,$title);
my (@array,@intensities,@bases);
my %lanes;

my $max = 0;
my $setmax = 0;

my $maxNumLanes=8;

my $width=3;
my ($file,$directory);
my @s;
my @total;
my $count;


sub draw_graphs {
    my ($lane,$tmpName,$outName,$title) = @_;
    open (OUT, ">$tmpName.txt") 
	|| die "Problem opening $tmpName.txt for writing: $!";

    print OUT << "END";
#    set terminal png color;
set terminal postscript portrait color solid;
#    set nokey;
set yrange [0:$max];
set ytics 500;
  
set size 1.0, 1.0;
set origin 0.0, 0.0;
#    set multiplot;
#    set size 1.0, 0.5;
#    set origin 0.0, 0.5
set title "$title";
set xlabel "Cycle Number" -1;
set ylabel "Intensity";
plot '$tmpName.dat' using 1:2 title 'A' with linespoints lt 9 lw $width pt 7 ps 2, '$tmpName.dat' using 1:3 title 'C' with linespoints lt 1 lw $width pt 7 ps 2,    '$tmpName.dat' using 1:4 title 'G' with linespoints lt 3 lw $width pt 7 ps 2, '$tmpName.dat' using 1:5 title 'T' with linespoints lt 2 lw $width pt 7 ps 2;  
  
#    set origin 0.0, 0.0;
#    set title "Occurrence of blank bases by cycle ";
#    set xlabel "Cycle Number" -1;
#    plot '$tmpName.dat' using 1:3 title 'blank rate' with impulses lt 3 lw 24;
#    set nomultiplot;

END
    close (OUT);
    gnuplotImage($tmpName.".txt", $outName);
    unlink "$tmpName.dat", "$tmpName.txt";
}

sub draw_graphs_percent {
    my ($lane,$tmpName,$outName,$title) = @_;
    open (OUT, ">$tmpName.txt") 
	|| die "Problem opening $tmpName.txt for writing: $!";

    print OUT << "END";
#    set terminal png color;
set terminal postscript portrait color solid;
#    set nokey;
set yrange [0:100];
set ytics 10;
 
set size 1.0, 1.0;
set origin 0.0, 0.0;
#    set multiplot;
#    set size 1.0, 0.5;
#    set origin 0.0, 0.5
set title "$title";
set xlabel "Cycle Number" -1;
set ylabel "Intensity";
  plot '$tmpName.dat' using 1:2 title 'A' with linespoints lt 9 lw $width pt 7 ps 2, '$tmpName.dat' using 1:3 title 'C' with linespoints lt 1 lw $width pt 7 ps 2,    '$tmpName.dat' using 1:4 title 'G' with linespoints lt 3 lw $width pt 7 ps 2, '$tmpName.dat' using 1:5 title 'T' with linespoints lt 2 lw $width pt 7 ps 2;    
#    set origin 0.0, 0.0;
#    set title "Occurrence of blank bases by cycle ";
#    set xlabel "Cycle Number" -1;
#    plot '$tmpName.dat' using 1:3 title 'blank rate' with impulses lt 3 lw 24;
#    set nomultiplot;

END
    close (OUT);
    gnuplotImage($tmpName.".txt", $outName);
    unlink "$tmpName.dat", "$tmpName.txt";
}

die "Usage: $0 Signal_Means.txt Directory_Name\n"
    unless (@ARGV==2);

$file = shift(@ARGV);
$directory = shift(@ARGV);

$directory.="/" unless ($directory=~/\\$/);

open (IN, $file) 
    || die "Problem opening $file: $!";
while (<IN>)
{
  
    next if($_ =~ m/\#/);
    push (@intensities,[split('\t',$_)]);

}
close(IN);


for($i=0;$i<scalar(@intensities);$i++){
    $lanes{$intensities[$i][0]}++;
}


### Going to start a loop for plotting graphs now


### Going to do call stuff first as will be highest intensities




### Going to do CALL 



for($i=1;$i<=$maxNumLanes;$i++){ ## fixed number of possible lanes
    
    if (exists $lanes{$i}){

	$outName = $directory."s_".$i."_call.png";
	$title = "s_".$i."_call.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";

	for($j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $i);

	    print  OUT "$intensities[$j][1]\t";
	    for($k=0;$k<4;$k++){
	 
		print  OUT "$intensities[$j][$k+6]\t";
		if($setmax == 0){
		
		    ### going to defined a max value based on first tile
		    if($intensities[$j][$k+6] > $max){
			$max = $intensities[$j][$k+6];
		    }	
		} 
	    }
	    print OUT "\n";
	}
	close (OUT);

	if($setmax == 0){
	    $max = $max * 1.5;
	    ## wanted to round to nearest 1000 so will cheat until i have more time
	    $max = $max / 1000;
	    $max = int($max);
	    $max = ($max+1) * 1000;
	    $setmax++;
	}
	draw_graphs($i,$tmpName,$outName,$title);
    }
}


### Going to do ALL 
for($i=1;$i<=$maxNumLanes;$i++){ ## fixed number of possible lanes
    
    if (exists $lanes{$i}){
	$outName = $directory."s_".$i."_all.png";
	$title = "s_".$i."_all.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";

	for($j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $i);

	    for($k=0;$k<5;$k++){
		print  OUT "$intensities[$j][$k+1]\t";
	    }
	    print OUT "\n";
	}
	close (OUT);
	draw_graphs($i,$tmpName,$outName,$title);
    }
}

### Going to do %ALL 
for($i=1;$i<9;$i++){ ## fixed number of possible lanes
   
    if (exists $lanes{$i}){

	$outName = $directory."s_".$i."_percent_all.png";
	$title = "s_".$i."_percent_all.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";

	for($j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $i);
	    $total = 0;
	    for($k=0;$k<4;$k++){
		$total+= $intensities[$j][$k+2];
	    }
	    print  OUT "$intensities[$j][1]\t";
	    for($k=0;$k<4;$k++){
		if($total != 0){
		    $intensities[$j][$k+2] =  $intensities[$j][$k+2] / $total * 100;
		}
		else {
		    $intensities[$j][$k+2] = 0;
		}   
		print  OUT "$intensities[$j][$k+2]\t";
	    }
	    print OUT "\n";
	}
	close (OUT);

	draw_graphs_percent($i,$tmpName,$outName,$title);
    }
}

### Going to do %call 
for($i=1;$i<9;$i++){ ## fixed number of possible lanes
    
    if (exists $lanes{$i}){

	$outName = $directory."s_".$i."_percent_call.png";
	$title = "s_".$i."_percent_call.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";

	for($j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $i);
	    $total = 0;
	    for($k=0;$k<4;$k++){
		$total+= $intensities[$j][$k+6];
	    }
	    print  OUT "$intensities[$j][1]\t";
	    for($k=0;$k<4;$k++){
		if($total != 0){
		    $intensities[$j][$k+6] =  $intensities[$j][$k+6] / $total * 100;
		}
		else {
		    $intensities[$j][$k+6] =  0;
		}	
		print  OUT "$intensities[$j][$k+6]\t";
	    }
	    print OUT "\n";
	}
	close (OUT);
	draw_graphs_percent($i,$tmpName,$outName,$title);
    }
}




### Going to do %Base Calls
for($i=1;$i<9;$i++){ ## fixed number of possible lanes
    
    if (exists $lanes{$i}){

	$outName = $directory."s_".$i."_percent_base.png";
	$title = "s_".$i."_percent_base.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";
    
	for($j=0;$j<scalar(@intensities);$j++){
	    next if ($intensities[$j][0] != $i);

	    print  OUT "$intensities[$j][1]\t";
	    for($k=0;$k<4;$k++){
		print  OUT "$intensities[$j][$k+10]\t";
	    }
	    print OUT "\n";
	}
	close (OUT);

	draw_graphs_percent($i,$tmpName,$outName,$title);
    }
}
