#!/usr/bin/env perl

#### This code plots % error rate curves for 0,1,2,3,4 errors per reads
#### The program inputs a realign file
####stores the numbers in a /tmp file to put the graph 
# Copyright (c) 2006 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

use warnings;
use strict;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;

#------------------------------------------------------------------------------

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plot_Error_Curves.pl,v 1.14 2007/09/18 10:05:27 rshaw Exp $';
    my $cvs_version='$Revision: 1.14 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope

die "Usage: $0 rescore.txt output.png\n"
    unless (@ARGV>=1);

my $name = shift(@ARGV);
my $outName;

# Take output name from command line arguments (recommended usage)
# Recommended usage is that output file name ends in .png, although
# ImageMagick convert can build other image file formats too
if (@ARGV==1)
{
    $outName=shift(@ARGV);
} # if
else
{ # make output name based on assumptions about input name 
  # (not recommended, kept for backwards compatibility)
    $outName = "$name";
    $outName =~ s/rescore/errors/;
    $outName =~ s/\.txt/\.png/;
    warn "No output filename specified, will send output to $outName\n";
} # else

die if ($outName eq $name);

#------------------------------------------------------------------------------

sub draw_graphs_impulses($$$$$)
{
    my $width = 24;
    my ($lane, $tmpName, $outName, $title, $label) = @_;

    open (OUT, ">$tmpName.txt") 
	|| die "Problem opening $tmpName.txt for writing: $!";

    print OUT << "END";
# set terminal png color;
set terminal postscript portrait color solid;
# set nokey;
set yrange [0:100];
set ytics 10;
    
set boxwidth 0.6;
    
set size 1.0, 1.0;
set origin 0.0, 0.0;
set title "$title";
set xlabel "Cycle Number" -1;
set ylabel "% reads with 'x' errors or less";
set key left bottom below
set label "$label" at -7,-7,0
# plot '$tmpName.dat' using 1:2 title '0 errors' with boxes lt 9 lw $width fs solid 0.2;

plot '$tmpName.dat'  using 1:6 title '4 or less' with impulses lt 7 lw $width, '$tmpName.dat' using 1:5 title '3 or less' with impulses lt 2 lw $width,'$tmpName.dat' using 1:4 title '2 or less' with impulses lt 3 lw $width,'$tmpName.dat' using 1:3 title '1 or less' with impulses lt 1 lw $width, '$tmpName.dat' using 1:2 title '0 errors' with impulses lt 9 lw $width; 


#plot '$tmpName.dat' using 1:2 title '0 errors' with boxes lt 9 lw $width fs solid 0.2, '$tmpName.dat' using 1:3 title '1 or less' with boxes lt 1 lw $width  fs solid 0.4,    '$tmpName.dat' using 1:4 title '2 or less' with boxes lt 3 lw $width  fs solid 0.6, '$tmpName.dat' using 1:5 title '3 or less' with boxes lt 2 lw $width  fs solid 0.8,'$tmpName.dat' using 1:6 title '4 or less' with boxes lt 7 lw $width  fs solid 1; 

END
    close (OUT);
    gnuplotImage($tmpName.".txt", $outName);
    unlink "$tmpName.dat", "$tmpName.txt";
}

#------------------------------------------------------------------------------

my @results;

open (IN, $name) 
    || die "Problem opening $name: $!";

my $in_cum_errs = 0;
my @num_unique_aligns_by_read;

my $cycle_num = 0;
my $num_cycles = 0;
my $bad_num_cycles_found = 0;

my $max_errors_to_count = 5; # default value - overridden by num cols found

my $read_num = 1; # default
my $num_reads = 1; # default

# @cycle_offsets is indexed by 1-offset $read_num.
# Initialise to zero for Read 1.
my @cycle_offsets = (0, 0);
my $cycle_offset = 0;

while (<IN>) {
    if ($_ =~ m/\#/) {
        if ($_ =~ /^\# Lane.*Read/) {
            if ($_ =~ /^\# Lane.* : Read (\d+)/) {
                # Header for numbered individual Read
                $num_reads = $read_num = $1; # assumes Reads in order

                if ($read_num > 1) {
		    if ($num_cycles == 0) {
			$bad_num_cycles_found = 1;
		    }

                    # We now know the number of cycles in the previous read.
                    updateCycleOffsets(\@cycle_offsets,
                                       $read_num - 1, $num_cycles);
                    $cycle_offset = $cycle_offsets[$read_num];
                }
            } else {
                # All Reads header - assumed after the individual Reads.
                last; # do not need any All Reads statistics
            }
        }

        next;
    }

    chomp;

    if ($_ =~ /unique alignments : (\d+) \(total score \d+\)/) {
	$num_unique_aligns_by_read[$read_num] = $1;
    } elsif ($_ =~ /cycles : (\d+)/) {
	$num_cycles = $1; # may be the only source 
    } elsif ($_ =~ /^\!(\d+)\s(.+)/) {
	$in_cum_errs = 1;

	$cycle_num = $1;
	my @err_counts = split('\t', $2);
	@{$results[$cycle_offset + $cycle_num - 1]} = @err_counts;

	$max_errors_to_count = @err_counts;
    } elsif ($in_cum_errs) {
        # First line after a cumulative errors line
	$in_cum_errs = 0;

	if (int($cycle_num) != int($num_cycles)) {
	    warn "Inconsistent numbers of cycles : $cycle_num, $num_cycles";
	}

	$num_cycles = $cycle_num; # assumes strictly increasing
    }
} # while

# We now know the number of cycles in the last individual Read.

if ($num_cycles == 0) {
    $bad_num_cycles_found = 1;
}

updateCycleOffsets(\@cycle_offsets, $read_num, $num_cycles);


if ($bad_num_cycles_found) {
    warn "No clusters found in one or more reads, creating blank $outName";
    open (OUT, ">$outName")
	|| die "Problem creating blank file $outName: $!";
    close(OUT);
    exit(0);
}

# Write gnuplot data file.
	
my $title = "$name";
$title =~ s/rescore/realign/; # to match old plots
my $tmpName="/tmp/tmp-plot_Error_Curves-$>-$$";

open (OUT, ">$tmpName.dat")
    || die "Problem opening $tmpName.dat for writing: $!";

for (my $read_ind = 1; $read_ind <= $num_reads; ++$read_ind) {
    my $num_unique_aligns = $num_unique_aligns_by_read[$read_ind];
    my $next_read_start_cycle = $cycle_offsets[$read_ind + 1];

    for (my $cycle_ind = $cycle_offsets[$read_ind];
         $cycle_ind < $next_read_start_cycle; ++$cycle_ind) {
   
        printf OUT "%d\t", $cycle_ind + 1;

        # If there are no unique alignments for a read, set count of reads
	# with < N errors to be 0 for all N considered.
        if ($num_unique_aligns == 0) {
	    for (my $num_errs_ind = 0; $num_errs_ind < $max_errors_to_count;
		 ++$num_errs_ind) {
		print OUT "0\t";
	    }

	    print OUT "\n";
	    next;
	}
  
        for (my $num_errs_ind = 0; $num_errs_ind < $max_errors_to_count;
             ++$num_errs_ind) {
            if (!defined($results[$cycle_ind][$num_errs_ind])) {
                print("result not defined for cycle-1 $cycle_ind, ",
                      "j $num_errs_ind\n");
            }

            $results[$cycle_ind][$num_errs_ind]
                = (($results[$cycle_ind][$num_errs_ind] / $num_unique_aligns)
                   * 100);
            print OUT "$results[$cycle_ind][$num_errs_ind]\t";
        }

        print OUT "\n";
    }
}

close(OUT);

my $lane = 1;

# Make label.
shift(@num_unique_aligns_by_read); # Removes undefined `read 0' value.
my $num_unique_aligns_str = join(', ', @num_unique_aligns_by_read);
my $label = "CLUSTERS = $num_unique_aligns_str";

draw_graphs_impulses($lane, $tmpName, $outName, $title, $label);

#------------------------------------------------------------------------------
