#!/usr/bin/env python
#
# Copyright (c) 2006 Solexa
# Author: Klaus Maisinger
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

import re, sys, os, getopt, math
import matrix

"""Reads in a list of matrix elements given by a flat file format
where each row contains lane, tile and 16 matrix elements. Selects a
median matrix per lane (defined as the matrix of the medians of the
matrix elements), and also a median matrix of all tiles."""

def usage():
    print "Usage: select_matrices.py <matrix summary filename>"
    
def parse_lines(lines):
    """Parse lines of a matrix summary file. The format is assumed 
to be lane<TAB>tile<TAB>elem11<SPACE>elem12<SPACE>elem13<SPACE>elem14
<TAB>..."""
    matrixlist = {}
    for line in lines:
        elements = line.split("\t")
        if (len(elements) == 6):
            try:
                lane = int(elements[0])
            except:
                sys.stderr.write("Warning: Lane index not integer.\n")
                continue
            matrix = []
            for element in elements[2:]:
                row = []
                try:
                    row = map(float, element.split())
                except:
                    sys.stderr.write("Warning: Matrix elements not floats\n")
                    continue
                if len(row) != 4:
                    sys.stderr.write("Warning: Not enough columns in matrix\n")
                    continue
                matrix += row
            if len(matrix) == 16:
                matrixlist.setdefault(lane, []).append(matrix)
            else:
                print matrix
                sys.stderr.write("Warning: Not enough rows in matrix. Skipping...\n")
        else:
            sys.stderr.write("Warning: File %s corrupted.\n" % filename)
    return matrixlist

if __name__ == "__main__":

    try:
        (options, args) = getopt.getopt(sys.argv[1:], "", [])
    except getopt.GetoptError:
        usage()
        sys.exit(1)
    if (len(args) != 1):
        usage()
        sys.exit(2)

    filename = args[0]
    if filename == "-":
        fd = sys.stdin
    else:
        try:
            fd = open(filename)
        except:
            sys.stderr.write("Failed to open %s\n" % filename)
            
    # read matrices and sort store them by lane
    lines = []
    try:
        lines = fd.readlines()
    except:
        sys.stderr.write("Failed to read %s\n" % filename)

    matrixlist = parse_lines(lines)

        
    # summarise all lanes
    matrices = []
    if len(matrixlist.values()) > 0:
        matrices = reduce(lambda x, y: x+y, matrixlist.values())

    # skip identity matrices
    # (don't bother to compare every single element)
    for i in range(len(matrices)-1, -1, -1):
        if (math.fabs(reduce(lambda x, y: math.fabs(x)+math.fabs(y),
                             matrices[i]) - 4.) <= 0.01):
            del matrices[i]

    if (len(matrices) <= 0):
        matrices = [ [ 0.]*16 ]
        for i in range(4):
            matrices[0][i*4+i] = 1.

    med = [ [ 0. ] * 4 for i in range(4) ]
    for i in range(4):
        for j in range(4):
            values = map(lambda x: x[i*4+j], matrices)
            values.sort()
            med[i][j] = values[len(values)/2]

    matrix.write_matrix(sys.stdout, med)

