#!/usr/bin/env python

# parse GERALD input file and update readlengths if necessary
#
# Copyright (c) Solexa 2005, 2006
# Author: Klaus Maisinger
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

import os, sys, string
import XMLSimple, common, runfolder

"""Parse Gerald config files and modify read-lengths."""

gerald_executable     = "GERALD.pl"
gerald_config         = "GERALD.xml"
toolspath             = os.getenv("TOOLS_DIR") or "../Gerald"
gerald_executable     = os.path.join(toolspath, gerald_executable)


class GeraldParams(common.Params):
    def __init__(self):
        self.geraldflag        = 0
        self.geraldconfigfiles = []
        self.options           = [ "GERALD=", "config=" ]

    def parse(self, options):
        for o, a in options:
            if o == "--GERALD" or o == "--config":
                path = os.path.abspath(a)
                if not os.path.exists(path):
                    common.error("Cannot find %s!" % path)
                    sys.exit(2)
                self.geraldconfigfiles.append(path)
                if o == "--GERALD":
                    self.geraldflag = 1


def get_my_path():
    return os.path.dirname(os.path.abspath(sys.argv[0]))


def getGeraldXML():
    data = {}
    filename = os.path.join(toolspath, gerald_config)
    filename = os.path.join(get_my_path(), filename)
    try:
        data = XMLSimple.XMLin(filename)
    except:
        common.error("Failed to read %s." % filename)
    return data
    
def getQhgInfo():
    data = getGeraldXML()
    pure_bases = 12
    use_bases = "all"
    if data.has_key('DefaultRunParameters'):
        if data['DefaultRunParameters'].has_key('PURE_BASES'):
            pure_bases = int(data['DefaultRunParameters']['PURE_BASES'])
        if data['DefaultRunParameters'].has_key('USE_BASES'):
            use_bases = data['DefaultRunParameters']['USE_BASES']
    return (pure_bases, use_bases)


def processGeraldConfig(runflag, geraldconfigs,
                        run_params, sequence_folder):
    ret_val = 0
    
    for i in range(len(geraldconfigs)):
        if (run_GERALD(geraldconfigs[i], run_params,
                       sequence_folder, runflag) != 0):
            ret_val = 1

    return ret_val


def run_GERALD(configfile, run_params, sequencefolder,
               runFlag = 0):
    """Run the GERALD makefile generator for sequence alignment."""
    commandline = "%s %s --EXPT_DIR %s" % \
                  (runfolder.get_executable(gerald_executable),
                   configfile, sequencefolder)
    if runFlag:
        commandline += " --FORCE"
    else:
        reads = []
        for read in run_params.Reads:
            reads.append("%i" % (read.LastCycle - read.FirstCycle + 1))
        commandline += " --length=%s" % ",".join(reads)
    print commandline

    # Prevent interleaving of GERALD.pl and other output.
    sys.stdout.flush()
    sys.stderr.flush()
    
    ret_val = os.system(commandline)

    sys.stdout.flush()
    sys.stderr.flush()

    #output = os.popen4(commandline)[1].read()
    #print output

    if (ret_val != 0):
        err_code = ret_val >> 8
        common.error("%s (%s) returned error code %d" % (gerald_executable,
                                                         configfile,
                                                         err_code))
        return 1
    else:
        return 0
                     
