#! /usr/bin/env python

#  Time-stamp: <17 Oct 2006 16:03 BST manteia.py km>
#
# Copyright (c) 2004, 2005, 2006 Solexa 
# Author: Klaus Maisinger
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

"""Utilities to analyse intensity series for cluster on Manteia
chips."""

import re, os, runfolder

class ManteiaFileName:
    """Manipulate and extract information such as the corresponding
    position on the chip and the incorporated base from a filename
    formed by the Genopsys software."""
    expr = re.compile("(\D{1,3}\d)(\d)(\D)(\d{3,5})(\D{2,4})\.(\D{1,3})")

    def match(filename):
        b = ManteiaFileName.expr.match(os.path.basename(filename))
        if b:
            return [ b.group(1), b.group(2), b.group(3), b.group(4),
                     b.group(5), b.group(6) ]
        return None
    match = staticmethod(match)

    def instrument():
        return "Manteia instrument"
    instrument = staticmethod(instrument)

    def is_deblocked(filename):
        d = ManteiaFileName.match(filename)[4]
        return (len(d) and d[0] == 's')
    is_deblocked = staticmethod(is_deblocked)

    def cmp(name1, name2):
        """Compare filenames without paths"""
        values1 = ManteiaFileName.match(name1)
        values2 = ManteiaFileName.match(name2)
        c=cmp("".join(values1[0:3]), "".join(values2[0:3]))
        if c != 0:
            return c
        return cmp(int(values1[3]), int(values2[3]))
    cmp = staticmethod(cmp)
        
    def tilestr(filename):
        matches = ManteiaFileName.match(filename)
        return "s_%i_%04i" % (int(matches[1]),
                              ManteiaFileName.tile(filename)+1)
        #return "".join(ManteiaFileName.match(filename)[0:3])
    tilestr = staticmethod(tilestr)

    def tile(filename):
        return (ord(ManteiaFileName.match(filename)[2]) - ord('a'))
    tile = staticmethod(tile)

    def lane(filename):
        return (int(ManteiaFileName.match(filename)[1]) - 1)
    lane = staticmethod(lane)

    def base(filename):
        return ManteiaFileName.match(filename)[4]
    base = staticmethod(base)

    def cycle(filename):
        #print filename, runfolder.CycleDirectory.cycle(os.path.split(filename)[0])
        return runfolder.CycleDirectory.cycle(os.path.split(filename)[0])
    cycle = staticmethod(cycle)


class ManteiaFilePath (ManteiaFileName):
    """Extend the Manteia filename class by using information from the
    runfolder directory name"""
    def cmp(name1, name2):
        folder1 = runfolder.get_bottom_level_folder(name1)
        folder2 = runfolder.get_bottom_level_folder(name2)
        comp = cmp(runfolder.CycleDirectory.cycle(folder1),
                   runfolder.CycleDirectory.cycle(folder2))
        if comp != 0 :
            return comp
        comp = cmp(runfolder.CycleDirectory.is_deblocked(folder1),
                   runfolder.CycleDirectory.is_deblocked(folder2))
        if comp != 0 :
            return comp
        return ManteiaFileName.cmp(name1, name2)
    cmp = staticmethod(cmp)


    
#
# some self-tests
#
if __name__ == "__main__":
    for samplefile in [ "/Cycle1/b11a299at.pck" ]:
        print "%s: tile %s, base %s, is deblocked: %d" % \
              (samplefile, ManteiaFileName.tile(samplefile),
               ManteiaFileName.base(samplefile),
               ManteiaFileName.is_deblocked(samplefile))
