
// NUMBAT.cpp
// Copyright (c) 2006 Solexa
// Author: A. J. Cox
// This source file is covered by the "Solexa Public Source License"
// agreement and bound by the terms therein.

#include <iostream>
#include <vector>
#include <algorithm>
#include <fstream>
#include <cmath>
#include <cassert>
#include <cctype>
#include <sys/time.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fstream>
#include <iostream>
#include <deque>
//#include <map>
//#include <dirent.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
//#include <string>
#include <ctime>
#include <algorithm>
#include "GlobalUtilities.h"

static char* id="@(#) $Id: NUMBAT.cpp,v 1.1 2006/07/07 18:21:33 tc Exp $";

// NUcleotide Methylated Batch Alignment Tool
// Match a batch of bisulphite-treated DNA tags against a set of
// genome files

using namespace std;
typedef unsigned int uint;




const uint readLength(25);
const uint segLength(readLength/2);
const uint maxNumNs(3);
const uint maxNumErrors(3);

// SeqFile: provide interface to a chromosome file. Assumes:
// First line might be a comment starting with '>', rest of lines are sequence.
// All sequence lines contain same number of bases, except possibly the last.
// All lines (comment+sequence) terminated by a single '>'
// NB Numbering of bases in file starts at 1.
class SeqFile
{
public:
  SeqFile( const char* name );
  ~SeqFile() { munmap( pMap, fileSize ); }

  char operator[]( unsigned int pos )
  {
    assert(pos<=numChars);
    --pos;
    pos+=(pos/lineLength);
    return pStart[pos];
  } // ~operator[]

  int getNumChars( void ) const { return numChars; }
private:
  void *pMap;
  const char* pStart;
  const char* pEnd;
  size_t fileSize;
  int lineLength;
  int numChars;
}; // ~class SeqFile

SeqFile::SeqFile( const char* name )
{
  int fd;
  // memory map sequence file
  fd = open(name, O_RDONLY, S_IRUSR );
  if (fd==-1)
  {
    cerr << "Error: could not open file " 
         << name << endl;
    exit (1);
  } // ~if
  fileSize = lseek(fd, 0, 2);
  cerr << "Opened file " << name << " of size "
       << fileSize << " bytes." << endl;

  pMap=mmap(0, fileSize, PROT_READ, MAP_SHARED, fd, 0);
  if (pMap==NULL)
  {
    cerr << "Error: could not memory map file " 
         << name << endl;
    exit (1);
  } // ~if
  close (fd);

  pStart=(char*)pMap;
  pEnd=pStart+fileSize;

  if (*pStart=='>') 
  {
    while (*pStart!='\n') ++pStart;
    ++pStart;
  } // ~if
  const char* pTemp(pStart);
  while (*pTemp!='\n') ++pTemp;
  lineLength=pTemp-pStart;
  cerr << "Assuming " << lineLength << " characters per line" << endl;

  numChars = 1 
    + (((pEnd-pStart-2)/(lineLength+1))*lineLength)
    + ((pEnd-pStart-2)%(lineLength+1));

  cerr << "Assuming " << numChars << " characters in total" << endl;


} // ~ctor



char titleBuf[256];


uint seqSize;


// NB scores are stored as shorts, so there will be problems 
// if max(defaultMatch, defaultMismatch)*seqSize>32767.
// defaultMatch=500 OK for read lengths<=65
// Assumed scoring scheme= 100*log10(odds of outcome);
const short defaultMatch=500;
const short defaultMismatch=-547;
// defaultMatch=500 is equivalent to correct:wrong=100000:1
// assuming the three possible wrong bases for any sequenced base
// are equiprobable, then the corresponding defaultMismatch is
// 100*log10(1/300000)=-547
const short defaultBlank=-47;
// "-47" comes from 100*log10(1/3) - log odds score if all four bases
// are equally likely to be sequenced as a blank

struct Seq
{
  //  Seq() {}
  char data[maxSeqSize];
}; // ~struct Seq

vector<Seq> readStore;

const Word notSet((Word)~0);

const Word keyMask ( ((Word)~0) >> ( (8*sizeof(Word))-(2*segLength) ));

struct KeySet
{
  KeySet( Word w ) : fwd(w), rev(notSet) {}
  Word fwd;
  Word rev;
  Word fwdM;
  Word revM;
}; // ~struct KeySet

const uint keyStoreSize(readLength-segLength+1);

const Word methMaskLow(0x55555555);

// methylate: change all Cs to Ts
Word methylate( Word w )
{
  return (w|((w&methMaskLow)<<1));
} // ~void methylate( Word& w )

union WordByte
{
  Word w;
  uchar uc[sizeof(Word)];
}; // ~union WordByte

// NB screws up for frag length not 12!!
Word reverseComp( const Word w )
{
  WordByte f,r;
  f.w=w;
  r.uc[0]=reverseChar[f.uc[2]];
  r.uc[1]=reverseChar[f.uc[1]];
  r.uc[2]=reverseChar[f.uc[0]];
  r.uc[3]=0;
  return r.w;
} // ~reverseComp

struct HashTable
{
  HashTable() : ptrs_(1<<(2*segLength),0), hits_(1) {}

  //  int numKeys;
  vector <uint> ptrs_;
  vector <vector<uint> > hits_;

  void hash (const Word key, const Word val)
  {
    if (ptrs_[key]==0)
    {
      ptrs_[key]=hits_.size();
      hits_.push_back( vector<uint>() );
    } // ~if
    hits_[ptrs_[key]].push_back(val);
    //  cout << "ht: hashing "; printWord (key, segLength); cout << " " << val << endl;
  } // ~hash
  vector<uint>* get (const Word key )
  {
    //    cout << "ht asking for key "; printWord(key,segLength); cout << endl;  
    return ((ptrs_[key]==0)?NULL:&hits_[ptrs_[key]]);
  }
}; // ~struct HashTable

HashTable h1, h2;

struct MatchInfo
{
  MatchInfo(): numBestMatches(0),bestNumErrors(99),frag(NULL),fileName(NULL)  {}
  uint numBestMatches;
  uint bestNumErrors;
  uint bestMatchPos;
  uint bestMatchFile;
  char bestMatchDir;
  char* frag;
  const char* fileName;
}; //~struct MatchInfo


vector <MatchInfo> match;
vector <uint> candidates;
char fragBuf[maxSeqSize];

void check( uint pos, uchar strand, const char* fileName )
{
  uint numErrors;
  //  MatchInfo* p;
  for (vector<uint>::iterator i(candidates.begin());i!=candidates.end();++i)
  {
    //    cout << "checking " << fragBuf << " vs " << *i << " " 
    //     << readStore[*i].data << endl;
    numErrors=0;
    for (uint j(0);j<readLength;j++)
    {
      numErrors+=(!((fragBuf[j]==readStore[*i].data[j])
                    ||((fragBuf[j]=='C')&&(readStore[*i].data[j]=='T'))));
      if (numErrors>maxNumErrors) break;
    }     
    //    cout << "helllo4" << endl;
    if (numErrors<=maxNumErrors)
    {
      // cout << *i << " " << numErrors << endl;
      //     pos=pos-readLength+1;
      if (numErrors<match[*i].bestNumErrors)
      {
        match[*i].numBestMatches=1;
        match[*i].bestNumErrors=numErrors;
        match[*i].bestMatchPos=pos-readLength+1;
        match[*i].bestMatchFile=9999;
        match[*i].bestMatchDir=strand;
        match[*i].fileName=fileName;
        match[*i].frag=new char[1+readLength];
        strncpy( match[*i].frag, fragBuf, readLength );
        match[*i].frag[readLength]='\0';
      }
      else if (numErrors==match[*i].bestNumErrors)
      {
        match[*i].numBestMatches++;
      }

    } // ~if
  } // ~for i
  // cout << "hello5" << endl;
  //  uint numBestMatches;
  //  uint bestNumErrors;
  //  uint bestMatchPos;
  // uint bestMatchFile;
  //  char bestMatchDir;


} // ~check



void check( const KeySet& k1, const KeySet& k2, SeqFile& chrom, const int pos,
            const char* fileName )
{
  vector <uint> *p1, *p2;

  candidates.clear();
  p1=h1.get(k1.fwdM);
  p2=h2.get(k2.fwdM);
  //  cout << pos << endl;
  //  printWord(k1.fwd,segLength);cout << " ";printWord(k2.fwd,segLength);cout << endl;
  //  printWord(k1.fwdM,segLength); cout << " "; printWord(k2.fwdM,segLength); cout << endl;

  //  printWord(k2.rev,segLength);cout << " ";printWord(k1.rev,segLength);cout << endl;
  //  printWord(k2.revM,segLength); cout << " "; printWord(k1.revM,segLength); cout << endl;

  // do forward strand
  if ((p1!=NULL)&&(p2!=NULL))
  {
    // cout << "fwd: found in both" << endl;
    for (vector<uint>::iterator i(p1->begin()); i!=p1->end(); ++i)
      // cout << *i << " - "; cout << endl;
    for (vector<uint>::iterator i(p2->begin()); i!=p2->end(); ++i)
      // cout << *i << " - "; cout << endl;
    candidates.resize(p1->size()+p2->size());
    vector<uint>::iterator endRange=
      set_union (p1->begin(),p1->end(),
                 p2->begin(),p2->end(),candidates.begin());
    //    for (vector<uint>::iterator i(candidates.begin()); i!=endRange; ++i)
      // cout << *i << " - ";
    // cout << endl;
    //   cout << "hello" << endl;
    candidates.resize(endRange-candidates.begin());
  }
  else if (p1!=NULL)
  {
    // cout << "fwd: found in h1" << endl;
    for (vector<uint>::iterator i(p1->begin()); i!=p1->end(); ++i)
      candidates.push_back(*i);
  }
  else if (p2!=NULL)
  {
    // cout << "fwd: found in h2" << endl;
    for (vector<uint>::iterator i(p2->begin()); i!=p2->end(); ++i)
      candidates.push_back(*i);
  }
  if (!candidates.empty())
  {
    //    cout << "hello2" << endl;
    for (int j(0); j<readLength; j++)
    {
      fragBuf[j]=toupper(chrom[pos-readLength+1+j]);
    }
    check( pos, 'F', fileName );
  }
  //    cout << "hello3" << endl;


  // do rev strand
  candidates.clear();
  p1=h1.get(k2.revM);
  p2=h2.get(k1.revM);
  if ((p1!=NULL)&&(p2!=NULL))
  {
    // cout << "rev: found in both" << endl;
    candidates.resize(p1->size()+p2->size());
    vector<uint>::iterator endRange=
      set_union (p1->begin(),p1->end(),
                 p2->begin(),p2->end(),candidates.begin());
    // for (vector<uint>::iterator i(candidates.begin()); i!=endRange; ++i)
      // cout << *i << " - ";
    // cout << endl;
    candidates.resize(endRange-candidates.begin());
  }
  else if (p1!=NULL)
  {
    // cout << "rev: found in h1" << endl;
    for (vector<uint>::iterator i(p1->begin()); i!=p1->end(); ++i)
      candidates.push_back(*i);
  }
  else if (p2!=NULL)
  {
    // cout << "rev: found in h2" << endl;
    for (vector<uint>::iterator i(p2->begin()); i!=p2->end(); ++i)
      candidates.push_back(*i);
  }
  if (!candidates.empty())
  {
    for (int j(0); j<readLength; j++)
    {
      fragBuf[j]=reverseCharASCII[chrom[pos-j]];
    }
    check( pos, 'R', fileName );
  }

}




int main( int numArgs, char* args [] )
{
  
  if (numArgs<4)
  {
    cerr << "syntax: " << args[0] 
	 << " [-useBases all|odd|YNYN... ] reads[.fa|.txt] -genome genomeFile1.fa [genomeFile2.fa ...] " << endl;

    exit (1);
  } // ~if

  int thisArg=0;
  int i;
  int j;
  int k;

  time_t timeNow = time(NULL);
  printf( "#RUN_TIME %s", ctime(&timeNow) );
  printf( "#SOFTWARE_VERSION %s\n", id );

  //  int genomeSize(0);
  //  vector<GenomeFrag> frags;
  //  vector<Seq> tempFragStore;
  // vector<Seq> fragStore;
  //  char* buf(NULL);
  // char* bufR(NULL);
  //  char* tagFileName(NULL);
  //  char* genomeFileName(NULL);
  char* basesToUse(NULL);

  fragBuf[readLength]='\0';

  ++thisArg;

  if (strcmp( args[thisArg], "-useBases")==0)
  {
    ++thisArg;
    assert(thisArg<numArgs);

    if (strcmp( args[thisArg], "all")==0)
    {
      // TBD need to delete basesToUse
      basesToUse=new char[readLength+1];
      memset(basesToUse,'Y',readLength);
      basesToUse[readLength]='\0';
    } // ~if
    else if (strcmp( args[thisArg], "odd")!=0)
    {
      // Assume custom useBases argument, error check it
      uint numYs(0), numNs(0);
      for (uint i(0); i<strlen(args[thisArg]); i++)
      {
        numYs+=(toupper(args[thisArg][i])=='Y');
        numNs+=(toupper(args[thisArg][i])=='N');
      } // ~for
      if (numYs+numNs<strlen(args[thisArg]))
      {
        cerr << "Error: could not parse -useBases argument " << args[thisArg]
             << endl
             << "Must be 'all', 'odd' or some combination of y,Y,n,N"
             << endl;
        exit (1);
      } // ~if
      if (numYs<readLength)

      {
        cerr << "Error: expecting " << readLength 
             << " bases, -useBases argument only specifies " << numYs << endl;
        exit (1);
      } // ~if

      basesToUse=new char[strlen(args[thisArg])+1];
      strcpy(basesToUse, args[thisArg]);
    } // ~else if
    ++thisArg;
    assert(thisArg<numArgs);
  } // ~if

  if (basesToUse==NULL)
  { // add in default (same result as '-useBases odd')
      basesToUse=new char[2*readLength];
      for (uint i(0); i<2*readLength-1; i++)
        basesToUse[i]=(((i&0x1)==0)?'Y':'N');
      basesToUse[(2*readLength)-1]='\0';
  } // ~if
    
  printf("#BASES_USED %s\n", basesToUse);


  ScoreSource* pScoresRaw(NULL);
  ScoreSourceFilter scoreFilter(basesToUse);
  ScoreSource* pScores(&scoreFilter);


  const char* scoreFileName(NULL);

  if (strcmp( args[thisArg], "-scoreFile")==0)
  {

    ++thisArg;
    assert(thisArg<numArgs);
    cerr << "Reading score information from file " << args[thisArg] 
	 << endl;

    scoreFileName=args[thisArg];

    printf( "#SCORE_FILE %s\n", args[thisArg] );

    ++thisArg;
    assert(thisArg<numArgs);

  } // ~if
  else
  {
    cerr << "Using default scores of " << defaultMatch 
	 << " for match, " << defaultMismatch << " for mismatch"
	 << endl;

    printf( "#SCORE_FILE none\n");
  } // ~else


  //  uint numIgnored(0);
  //  uint seqNum(0);

  OligoSource* pOligosRaw(NULL);
  OligoSourceFilter oligoFilter(basesToUse);
  OligoSource* pOligos(&oligoFilter);

  const char* strBuf;

  //  vector<QuerySeq> seqs;
  // QuerySeqs seqs_(readLength);
  //  HashTable h1, h2;
  
  deque <KeySet> keyStore;
  //  char thisChar;
  Word thisCode;
  Word thisKey;
  Word thisKey2;
  int numNs;

  for (i=thisArg; i<numArgs; i++)
  { 
    cerr << i << endl;
    if (strcmp( args[i], "-genome")==0) { cerr <<"hello" << endl; break;}

    pOligosRaw=OligoSource::getOligoSource(args[thisArg]);

    cerr << "Opened sequence file " << args[i] << endl; 
    printf ("#SEQ_FILE %s\n", args[i]);

    if (scoreFileName!=NULL)
    {
      oligoFilter.link(pOligosRaw);
      pScoresRaw=new ScoreSourceCycle( *pOligos, scoreFileName );
      pScores=pScoresRaw;
      //      pScoresRaw=new ScoreSourceCycle( *pOligosRaw, scoreFileName );
      printf ("#SCORE_FILE %s\n", scoreFileName);
    } // ~if
    else
    {
      pScoresRaw=dynamic_cast<ScoreSource*>(pOligosRaw);
      if (pScoresRaw!=NULL)
      {
        printf ("#SCORE_FILE %s\n", args[i]);
        scoreFilter.link(pScoresRaw);
        oligoFilter.link(pOligosRaw);
        pScores=&scoreFilter;
        pOligos=&oligoFilter;
      } // ~if
      else 
      {
        //        pScoresRaw=new ScoreSourceBasic
        pScoresRaw=new ScoreSourceBasic
          ( *pOligos, defaultMatch, defaultBlank, defaultMismatch );
        pScores=pScoresRaw;
        oligoFilter.link(pOligosRaw);
        pOligos=&oligoFilter;
        printf ("#SCORE_FILE none\n");
      } // ~else
    } // ~else

    // pScoresRaw and pOligosRaw never accessed directly from now on
    // ...until they are no longer needed and are deleted

    while ((strBuf=pOligos->getNextOligo())!=NULL)
    {
      readStore.push_back( Seq() );
      strncpy( readStore.back().data, strBuf, readLength );
      for (j=0;j<readLength;j++)
        readStore.back().data[j]=toupper(readStore.back().data[j]);
        
      thisKey=0;
      thisKey2=0;
      numNs=0;
      for (j=0;j<segLength;j++)
      {
        //        readStore.back().data[j]=toupper(readStore.back().data[j])
        thisCode=whichBase[readStore.back().data[j]];
        if (thisCode==nv)
        {
          thisCode=0; numNs++;
        }
        thisKey<<=2;
        thisKey|=thisCode;
      }
      for (j=readLength-segLength;j<readLength;j++)
      {
        thisCode=whichBase[readStore.back().data[j]];
        if (thisCode==nv)
        {
          thisCode=0; numNs++;
        }
        thisKey2<<=2;
        thisKey2|=thisCode;
      }

      if (numNs<=maxNumNs)
      {
        h1.hash( methylate(thisKey), readStore.size()-1 );
        h2.hash( methylate(thisKey2), readStore.size()-1 );
      }
      // cout << readStore.back().data << endl;

      //      seqs_.addSeq( *pOligos, *pScores );
      //  seqs.push_back( QuerySeq( *pOligos, *pScores, readLength, seqNum++));
      //          cout << seqs[seqNum-1].num_ << " " << seqs[seqNum-1].maxScore_ 
      //    << seqs[seqNum-1].data_ << endl;

      // TBD skip prefix + ignore blanks

    } // ~while

    if(dynamic_cast<ScoreSource*>(pOligosRaw)==NULL) delete pScoresRaw;
    delete pOligosRaw;

    //    fclose(pSeq);
  } // ~for i

  match.resize(readStore.size());

  assert (i<numArgs);
  assert (strcmp( args[i], "-genome")==0);
  thisArg=i+1;

  
  for (i=thisArg; i<numArgs;i++)
  {
    SeqFile chrom(args[i]);
    const int chromSize(chrom.getNumChars());    

    for ( j=1; j<=chromSize; j++ )
    {
      //      cout << j << " ";
      if (keyStore.empty())
      {
        // Try to fill in first key in keyStore
        thisKey=0;
        // if key contains duff characters skip past them
        // start at end and work back so we can skip as far as possible
        // as soon as possible
        //   cout << "first elt ";
        k=segLength-1;
        if ((j+k)>chromSize) break;
        for (;k>=0;k--)
        {
          thisCode=whichBase[chrom[j+k]];
          if (thisCode==nv) break;
          thisKey|=(thisCode<<(2*(segLength-1-k)));
        } // ~if
        if (thisCode==nv)
        { 
          j+=k; 
          //  cout << "skipping to " << j << endl;
          continue;
        } // ~if
        else
        {
          j+=segLength-1;
          // got first key, yippee
          keyStore.push_back( KeySet(thisKey) );
          //   printWord ( thisKey, segLength); cout << endl;
        } // ~else
      }
      else
      { 
        thisCode=whichBase[chrom[j]];
        if (thisCode!=nv)
        {
          thisKey=keyStore.back().fwd;
          thisKey<<=2;
          thisKey|=thisCode;
          thisKey&=keyMask;
          keyStore.push_back( KeySet(thisKey) );
          //  cout << "hash "; printWord ( thisKey, segLength);
          if (keyStore.size()==keyStoreSize)
          {
            // cout << "check ";
            // key store is ready to go
            keyStore.back().rev=reverseComp(keyStore.back().fwd);
            keyStore.back().fwdM=methylate(keyStore.back().fwd);
            keyStore.back().revM=methylate(keyStore.back().rev);
            //   printWord( keyStore.back().fwd,16); cout << " ";
            //  printWord( keyStore.back().rev,16); cout << endl;
            if (keyStore.front().rev==notSet)
            {
              //              cout << "rc ";
              // if rev comp was not filled in for first key, then do so
              keyStore.front().rev=reverseComp(keyStore.front().fwd);
              keyStore.front().fwdM=methylate(keyStore.front().fwd);
              keyStore.front().revM=methylate(keyStore.front().rev);
            } // ~if

            check (keyStore.front(),keyStore.back(),chrom,j,args[i]);
            // Now done with last element in queue, dump it
            keyStore.pop_front();
          }
        }
        else
        {
          //          cout << "duff ";
          // this key contained a duff character, clear store 
          keyStore.clear();
        }
      } // ~else  
      //            cout << endl;
    } // ~for j
    keyStore.clear();
    
  } // ~for i

  for (i=0;i<readStore.size();i++)
  {
    printf ("%s\t%d\t%d", readStore[i].data, 
            match[i].bestNumErrors, match[i].numBestMatches );
    if (match[i].numBestMatches==1)
    {
      assert(match[i].frag!=NULL);
      assert(match[i].fileName!=NULL);
      printf ("\t%d\t%c\t%s\t%s", match[i].bestMatchPos, match[i].bestMatchDir,
              match[i].frag, match[i].fileName);
      
    }
    printf ("\n");
      
  }

}
