// -*- c++ -*-
/*****************************************************************************/
// Copyright (c) Illumina 2007
// Author: Richard Shaw
// This source file is covered by the "Solexa Public Source License"
// agreement and bound by the terms therein.
/*****************************************************************************/
//
// This class can be used to read parameter value data from a file in chunks,
// each of which will be returned as a DataSet object.
//
// The file format consists of a variable number of columns containing the
// parameters. Each row corresponds to one base-call. The final two columns
// contain the called and aligned base respectively.
//
/*****************************************************************************/

#ifndef DATA_SET_FACTORY_H
#define DATA_SET_FACTORY_H

/*****************************************************************************/

#include <string>
#include <fstream>

#include "ReadParams.h"

/*****************************************************************************/

class Data_Set_Factory
{
 public:
    Data_Set_Factory(const std::string& data_file_name);
    ~Data_Set_Factory();

    /** This method allocates the DataSet. Caller must delete it.
        Only DataSet::MAX_NUM_RECORDS file lines will be parsed per call.
    */
    bool get_next_data_set(DataSet*& data_set_ptr, bool& at_eof);

 private:
    std::string my_data_file_name;
    std::ifstream my_data_file;
};

/*****************************************************************************/

#endif // ! DATA_SET_FACTORY_H

/*****************************************************************************/
