/*
 * Copyright (c) Solexa 2006
 * Author: Klaus Maisinger
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

/*
 * lookup table to hold the binned raw data set.
 */

#ifndef DATATABLE_H_
#define DATATABLE_H_

#include "ReadParams.h"
#include "BinnedTable.h"

class ErrorCount {
public:
    long count;
    long error;
    ErrorCount() { count = 0; error = 0; };
    virtual ~ErrorCount() {};
};

/*
 * This lookup table splits the errors by type; there are 4*4 different 
 * possibilities for a base to be misread as another base (for example A -> C).
 * This is not actually used so far; something along these lines may be useful
 * at some stage to distinguish different error types. For example,
 * because of the nature of the laser/dye/filter combinations, A and C
 * are more likely to be confused in the current Solexa system than say 
 * A and G, or A and T.
 */
template <class T>
class ErrorLookup : public std::vector<T>
{
public:
    ErrorLookup() {};
    virtual ~ErrorLookup() {};
    virtual int getErrorIndex(const BaseAlignment &alignment)
    {
        char bases[4] = { 'A', 'C', 'G', 'T' };
        int i;
        int idx1 = -1, idx2 = -1;
        for (i = 0; i < 4; i++)
            if (alignment.base_read == bases[i]) {
                 idx1 = i; break;
            }
        if (idx1 == -1)
             return -1;
        for (i = 0; i < 4; i++)
            if (alignment.base_aligned == bases[i]) {
                idx2 = i; break;
            }
        if (idx2 == -1)
            return -1;
        return (4*idx1+idx2);
    }
};

/*
 * Table to hold the binned input data.
 */
class DataTable : public BinnedTable<ErrorLookup<ErrorCount> >
{
public:
    DataTable();
    virtual ~DataTable();
    virtual void create();
    virtual void addData(DataSet &set);
    virtual void print();
    virtual ErrorCount getErrorCount(int idx)
    {
        assert(idx < table.size());
        assert(table[idx].size() == 16);
        int i;
        ErrorCount count;        
        for (i = 0; i < table[idx].size(); i++) {
            count.count += table[idx][i].count;
            count.error += table[idx][i].error;
        }
        return count;
    }
};

#endif /*DATATABLE_H_*/
