// -*- c++ -*-
/*****************************************************************************/
// Copyright (c) Illumina 2007
// Author: Richard Shaw
// This source file is covered by the "Solexa Public Source License"
// agreement and bound by the terms therein.
/*****************************************************************************/

#ifndef XMLCONFIGHANDLER_H
#define XMLCONFIGHANDLER_H

/*****************************************************************************/

#include <iostream>
#include <string>
#include <list>
#include <map>

#include "misc_types.h"
#include "XMLContentHandler.h"

/*****************************************************************************/

class Parameter_Config {
public:
    Parameter_Config();
    void reset();
    bool set_item_val(const std::string& item_name,
                      const std::string& val_str);
    bool is_ready() const;
    bool is_valid() const;
    const std::string& name() const { return my_name; }
    bool get_bin_boundary_vec(Float_Vec& bin_boundary_vec) const;

    friend std::ostream& operator<<(std::ostream& ostrm, 
                                    const Parameter_Config& param_cfg);

private:
    typedef enum { SPEC_TYPE_UNKNOWN             = 0x00,
                   SPEC_TYPE_BOUNDARY_LIST       = 0x01,  
                   SPEC_TYPE_FIRST_LAST_NUM_BINS = 0x02,
                   SPEC_TYPE_ANY                 = 0xff }; // bit masks

    typedef enum { TYPE_STRING, TYPE_INT, TYPE_FLOAT,
                   TYPE_FLOAT_LIST } Item_Type;
    struct Item_Info {
        Item_Type my_type;
        unsigned int my_spec_mask;
        bool my_defined_flag;

        Item_Info() 
            : my_type(TYPE_STRING), my_defined_flag(false),
              my_spec_mask(SPEC_TYPE_UNKNOWN) { }
        Item_Info(Item_Type item_type, unsigned int spec_mask)
            : my_type(item_type), my_defined_flag(false),
              my_spec_mask(spec_mask) { }
    };

    typedef std::map<std::string, Item_Info> Item_Name_Info_Map;
    typedef Item_Name_Info_Map::iterator Item_Name_Info_Map_Iter;
    typedef Item_Name_Info_Map::const_iterator Item_Name_Info_Map_CIter;

    Item_Name_Info_Map my_item_name_info_map;

    std::string my_name;
    unsigned int my_spec_type;
    Float_Vec my_bin_boundary_vec;
    float my_first_bin_low_bound;
    float my_first_bin_high_bound;
    float my_last_bin_low_bound;
    float my_last_bin_high_bound;
    unsigned int my_num_bins;
};

std::ostream& operator<<(std::ostream& ostrm, 
                         const Parameter_Config& param_cfg);

typedef std::list<Parameter_Config> Parameter_Config_List;
typedef Parameter_Config_List::const_iterator Parameter_Config_List_CIter;

/*****************************************************************************/

class XMLConfigHandler : public XMLContentHandler
{
public:
    XMLConfigHandler();
    virtual ~XMLConfigHandler() {};
    virtual void startDocument();
    virtual void startElement(const char name[], const Attributes *attr = 0);
    virtual void endElement(const char name[]);
    virtual void endDocument();
    virtual void characters(const char ch[], int start, int length);

    bool get_param_cfg_list(Parameter_Config_List& param_cfg_list) const;

private:
    bool my_parse_error_flag;
    bool my_in_param_flag;
    std::string my_curr_ele_name;
    Parameter_Config_List my_param_cfg_list;
    Parameter_Config my_curr_param_cfg;
};

/*****************************************************************************/

#endif // ! XMLCONFIGHANDLER_H

/*****************************************************************************/
